/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_MATMULIMPL_H_
#define AIDGE_CPU_OPERATOR_MATMULIMPL_H_

#include <array>
#include <memory>
#include <vector>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/MatMul.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"

namespace Aidge {
// class MatMul_Op;

// compute kernel registry for forward and backward
class MatMulImplForward_cpu
    : public Registrable<MatMulImplForward_cpu, std::tuple<DataType, DataType, DataType>,
                         void(const MatMul_Op::Attrs &, const DimSize_t, const DimSize_t,
                              const void *, const void *, void *)> {};
class MatMulImplBackward_cpu
    : public Registrable<MatMulImplBackward_cpu, std::tuple<DataType, DataType, DataType>,
                         void(const MatMul_Op::Attrs &, const DimSize_t, const DimSize_t,
                              const void *, const void *, void *)> {};

class MatMulImpl_cpu : public OperatorImpl {
private:
    const MatMul_Op &mOp;
    std::array<NbElts_t, 2> mNbConsumedData;
    std::array<NbElts_t, 1> mNbProducedData;

public:
    MatMulImpl_cpu(const MatMul_Op &op)
        : mOp(op),
          mNbConsumedData({0, 0}),
          mNbProducedData({0})
        {
            // ctor
        }

    static std::unique_ptr<MatMulImpl_cpu> create(const MatMul_Op &op)
    {
        return std::make_unique<MatMulImpl_cpu>(op);
    }

public:
    NbElts_t getNbRequiredData(const IOIndex_t inputIdx) const override final;
    NbElts_t getNbRequiredProtected(const IOIndex_t inputIdx) const override final;
    NbElts_t getRequiredMemory(const IOIndex_t /*outputIdx*/,
                               const std::vector<DimSize_t> & /*inputsSize*/) const override final;
    NbElts_t getNbConsumedData(const IOIndex_t inputIdx) const override final;
    NbElts_t getNbProducedData(const IOIndex_t outputIdx) const override final;

    void updateConsummerProducer() override final;

    void forward() override;
    void backward() override;
};

namespace {
static Registrar<MatMul_Op> registrarMatMulImpl_cpu("cpu", Aidge::MatMulImpl_cpu::create);
}
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_MATMULIMPL_H_ */
